﻿using System;
using Fadd.Messaging;

namespace Fadd.IO.Consoles
{
    /// <summary>
    /// Response generated by a command
    /// </summary>
    [Serializable]
    public class CommandResponse : IMessage
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ConsoleOutput"/> class.
        /// </summary>
        /// <param name="color">The color.</param>
        /// <param name="text">The text.</param>
        public CommandResponse(ConsoleColor color, string text)
        {
            Color = color;
            Text = text;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ConsoleOutput"/> class.
        /// </summary>
        /// <param name="text">The text.</param>
        public CommandResponse(string text)
        {
            Text = text;
        }

        /// <summary>
        /// Gets or sets console color.
        /// </summary>
        public ConsoleColor Color { get; set; }

        /// <summary>
        /// Gets or sets output text.
        /// </summary>
        public string Text { get; set; }


        public static ConsoleOutput Error(string message)
        {
            return new ConsoleOutput(ConsoleColor.Red, message);
        }

    }
}