﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using Fadd.Logging.Filters;
using Fadd.Logging.Targets;

namespace Fadd.Logging
{
    public class Logger : ILogger
    {
        private readonly IEnumerable<ILogFilter> _filters;
        private readonly IEnumerable<ILogTarget> _targets;

        public Logger(IEnumerable<ILogFilter> filters,  IEnumerable<ILogTarget> targets)
        {
            _filters = filters;
            _targets = targets;
        }

        public void Debug(string message)
        {
            WriteEntry(LogLevel.Debug, message, null);
        }

        public void Debug(string message, Exception ex)
        {
            WriteEntry(LogLevel.Debug, message, ex);
        }

        public void Info(string message)
        {
            WriteEntry(LogLevel.Info, message, null);
        }

        public void Info(string message, Exception ex)
        {
            WriteEntry(LogLevel.Info, message, ex);
        }

        public void Warning(string message)
        {
            WriteEntry(LogLevel.Warning, message, null);
        }

        public void Warning(string message, Exception ex)
        {
            WriteEntry(LogLevel.Warning, message, ex);
        }

        public void Error(string message)
        {
            WriteEntry(LogLevel.Error, message, null);
        }



        public void Error(string message, Exception ex)
        {
            WriteEntry(LogLevel.Error, message, ex);
        }

        private void WriteEntry(LogLevel level, string message, Exception exception)
        {
            var frames = new StackTrace(2).GetFrames();
            if (frames != null)
                frames = frames.Take(5).ToArray();

            var userName = Thread.CurrentPrincipal.Identity.Name;
            if (string.IsNullOrEmpty(userName))
                userName = Environment.UserName;

            var entry = new LogEntry
            {
                CreatedAt = DateTime.Now,
                Exception = exception,
                LogLevel = level,
                Message = message,
                StackFrames = frames,
                ThreadId = Thread.CurrentThread.ManagedThreadId,
                UserName = userName
            };

            // find any filter that says no to logging
            if (_filters.Any(filter => !filter.CanLog(entry)))
            {
                Console.WriteLine("Filter blocks");
                return;
            }

            _targets.Each(e => e.Enqueue(entry));
        }

    }
}
