﻿using System;
using System.Net;
using System.Net.Sockets;

namespace Fadd.Remoting
{
	/// <summary>
	/// Hosting remoting commands.
	/// </summary>
	public class RemotingHost
	{
		private IPEndPoint _endPoint;
		private readonly IServiceResolver _serviceResolver;
		private TcpListener _listener;

		/// <summary>
        /// Initializes a new instance of the <see cref="RemotingHost"/> class.
		/// </summary>
		/// <param name="components">Used to resolve services.</param>
        public RemotingHost(IServiceResolver components)
		{
			_serviceResolver = components;
		}

		/// <summary>
		/// Start channel
		/// </summary>
		/// <param name="value">End point to connect to or listen on.</param>
		public void Start(IPEndPoint value)
		{
			_endPoint = value;
			StartListener(value);
		}

		private void StartListener(IPEndPoint value)
		{
			_listener = new TcpListener(value);
			_listener.Start();
			_listener.BeginAcceptSocket(OnAcceptClient, null);
		}

		private void OnAcceptClient(IAsyncResult ar)
		{
			try
			{
				Socket socket = _listener.EndAcceptSocket(ar);
                RemotingClient channel = new RemotingClient(_serviceResolver, socket);
			}
			catch(Exception err)
			{
				UnhandledException.TriggerEvent(this, "Failed to end accept", err);
			}

			try
			{
				_listener.BeginAcceptSocket(OnAcceptClient, null);
			}
			catch(Exception err)
			{
			    UnhandledException.TriggerEvent(this, "Failed to start accepting a new client.", err);
			}

		}

		public void Stop()
		{
			if (_listener != null)
			{
				_listener.Stop();
				_listener = null;
			}			
		}

	}
}
